/****************************************************************************
** Meta object code from reading C++ file 'controlwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/widgets/controlwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'controlwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14FloatingButtonE_t {};
} // unnamed namespace

template <> constexpr inline auto FloatingButton::qt_create_metaobjectdata<qt_meta_tag_ZN14FloatingButtonE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "FloatingButton",
        "requestShowMenu",
        "",
        "requestShowTips",
        "requestHideTips"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestShowMenu'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestShowTips'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestHideTips'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<FloatingButton, qt_meta_tag_ZN14FloatingButtonE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject FloatingButton::staticMetaObject = { {
    QMetaObject::SuperData::link<DFloatingButton::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14FloatingButtonE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14FloatingButtonE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14FloatingButtonE_t>.metaTypes,
    nullptr
} };

void FloatingButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<FloatingButton *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestShowMenu(); break;
        case 1: _t->requestShowTips(); break;
        case 2: _t->requestHideTips(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (FloatingButton::*)()>(_a, &FloatingButton::requestShowMenu, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (FloatingButton::*)()>(_a, &FloatingButton::requestShowTips, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (FloatingButton::*)()>(_a, &FloatingButton::requestHideTips, 2))
            return;
    }
}

const QMetaObject *FloatingButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *FloatingButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14FloatingButtonE_t>.strings))
        return static_cast<void*>(this);
    return DFloatingButton::qt_metacast(_clname);
}

int FloatingButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = DFloatingButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void FloatingButton::requestShowMenu()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void FloatingButton::requestShowTips()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void FloatingButton::requestHideTips()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
namespace {
struct qt_meta_tag_ZN13ControlWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto ControlWidget::qt_create_metaobjectdata<qt_meta_tag_ZN13ControlWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "ControlWidget",
        "requestSwitchUser",
        "",
        "std::shared_ptr<User>",
        "user",
        "requestShutdown",
        "requestSwitchSession",
        "session",
        "requestTogglePopup",
        "QPoint",
        "globalPos",
        "QWidget*",
        "popup",
        "requestHidePopup",
        "addModule",
        "dss::module::BaseModuleInterface*",
        "module",
        "setUserSwitchEnable",
        "visible",
        "setSessionSwitchEnable",
        "chooseToSession",
        "setKBLayoutVisible",
        "setKeyboardType",
        "str",
        "setKeyboardList",
        "onItemClicked",
        "showSessionPopup",
        "showUserListPopupWidget",
        "showInfoTips",
        "hideInfoTips"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'requestSwitchUser'
        QtMocHelpers::SignalData<void(std::shared_ptr<User>)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'requestShutdown'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'requestSwitchSession'
        QtMocHelpers::SignalData<void(const QString &)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Signal 'requestTogglePopup'
        QtMocHelpers::SignalData<void(QPoint, QWidget *)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 9, 10 }, { 0x80000000 | 11, 12 },
        }}),
        // Signal 'requestHidePopup'
        QtMocHelpers::SignalData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'addModule'
        QtMocHelpers::SlotData<void(dss::module::BaseModuleInterface *)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 15, 16 },
        }}),
        // Slot 'setUserSwitchEnable'
        QtMocHelpers::SlotData<void(const bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'setSessionSwitchEnable'
        QtMocHelpers::SlotData<void(const bool)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 18 },
        }}),
        // Slot 'chooseToSession'
        QtMocHelpers::SlotData<void(const QString &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 7 },
        }}),
        // Slot 'setKBLayoutVisible'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setKeyboardType'
        QtMocHelpers::SlotData<void(const QString &)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'setKeyboardList'
        QtMocHelpers::SlotData<void(const QStringList &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 23 },
        }}),
        // Slot 'onItemClicked'
        QtMocHelpers::SlotData<void(const QString &)>(25, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 23 },
        }}),
        // Slot 'showSessionPopup'
        QtMocHelpers::SlotData<void()>(26, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showUserListPopupWidget'
        QtMocHelpers::SlotData<void()>(27, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'showInfoTips'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'hideInfoTips'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ControlWidget, qt_meta_tag_ZN13ControlWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject ControlWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ControlWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ControlWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN13ControlWidgetE_t>.metaTypes,
    nullptr
} };

void ControlWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ControlWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->requestSwitchUser((*reinterpret_cast<std::add_pointer_t<std::shared_ptr<User>>>(_a[1]))); break;
        case 1: _t->requestShutdown(); break;
        case 2: _t->requestSwitchSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->requestTogglePopup((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[2]))); break;
        case 4: _t->requestHidePopup(); break;
        case 5: _t->addModule((*reinterpret_cast<std::add_pointer_t<dss::module::BaseModuleInterface*>>(_a[1]))); break;
        case 6: _t->setUserSwitchEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 7: _t->setSessionSwitchEnable((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 8: _t->chooseToSession((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->setKBLayoutVisible(); break;
        case 10: _t->setKeyboardType((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 11: _t->setKeyboardList((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 12: _t->onItemClicked((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 13: _t->showSessionPopup(); break;
        case 14: _t->showUserListPopupWidget(); break;
        case 15: _t->showInfoTips(); break;
        case 16: _t->hideInfoTips(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QWidget* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ControlWidget::*)(std::shared_ptr<User> )>(_a, &ControlWidget::requestSwitchUser, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ControlWidget::*)()>(_a, &ControlWidget::requestShutdown, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ControlWidget::*)(const QString & )>(_a, &ControlWidget::requestSwitchSession, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ControlWidget::*)(QPoint , QWidget * )>(_a, &ControlWidget::requestTogglePopup, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ControlWidget::*)()>(_a, &ControlWidget::requestHidePopup, 4))
            return;
    }
}

const QMetaObject *ControlWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ControlWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN13ControlWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ControlWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void ControlWidget::requestSwitchUser(std::shared_ptr<User> _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void ControlWidget::requestShutdown()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ControlWidget::requestSwitchSession(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void ControlWidget::requestTogglePopup(QPoint _t1, QWidget * _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void ControlWidget::requestHidePopup()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}
QT_WARNING_POP
