/****************************************************************************
** Meta object code from reading C++ file 'displayinterface.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../src/global_util/dbus/displayinterface.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'displayinterface.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN16DisplayInterfaceE_t {};
} // unnamed namespace

template <> constexpr inline auto DisplayInterface::qt_create_metaobjectdata<qt_meta_tag_ZN16DisplayInterfaceE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "DisplayInterface",
        "BrightnessChanged",
        "",
        "BuiltinOutputChanged",
        "DisplayModeChanged",
        "HasChangedChanged",
        "MonitorsChanged",
        "PrimaryChanged",
        "PrimaryRectChanged",
        "ScreenHeightChanged",
        "ScreenWidthChanged",
        "__propertyChanged__",
        "QDBusMessage",
        "msg",
        "Apply",
        "QDBusPendingReply<>",
        "AssociateTouchScreen",
        "in0",
        "in1",
        "ChangeBrightness",
        "JoinMonitor",
        "ListOutputNames",
        "QDBusPendingReply<QStringList>",
        "QueryCurrentPlanName",
        "QDBusPendingReply<QString>",
        "QueryOutputFeature",
        "QDBusPendingReply<int>",
        "Reset",
        "ResetBrightness",
        "ResetChanges",
        "SaveChanges",
        "SetBrightness",
        "SetPrimary",
        "SplitMonitor",
        "SwitchMode",
        "Brightness",
        "BrightnessMap",
        "BuiltinOutput",
        "QDBusObjectPath",
        "DisplayMode",
        "HasChanged",
        "Monitors",
        "QList<QDBusObjectPath>",
        "Primary",
        "PrimaryRect",
        "DisplayRect",
        "ScreenHeight",
        "ScreenWidth"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'BrightnessChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'BuiltinOutputChanged'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'DisplayModeChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'HasChangedChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'MonitorsChanged'
        QtMocHelpers::SignalData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'PrimaryChanged'
        QtMocHelpers::SignalData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'PrimaryRectChanged'
        QtMocHelpers::SignalData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ScreenHeightChanged'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'ScreenWidthChanged'
        QtMocHelpers::SignalData<void()>(10, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot '__propertyChanged__'
        QtMocHelpers::SlotData<void(const QDBusMessage &)>(11, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 12, 13 },
        }}),
        // Slot 'Apply'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(14, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'AssociateTouchScreen'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &)>(16, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'ChangeBrightness'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, double)>(19, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 17 }, { QMetaType::Double, 18 },
        }}),
        // Slot 'JoinMonitor'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, const QString &)>(20, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 17 }, { QMetaType::QString, 18 },
        }}),
        // Slot 'ListOutputNames'
        QtMocHelpers::SlotData<QDBusPendingReply<QStringList>()>(21, 2, QMC::AccessPublic, 0x80000000 | 22),
        // Slot 'QueryCurrentPlanName'
        QtMocHelpers::SlotData<QDBusPendingReply<QString>()>(23, 2, QMC::AccessPublic, 0x80000000 | 24),
        // Slot 'QueryOutputFeature'
        QtMocHelpers::SlotData<QDBusPendingReply<int>(const QString &)>(25, 2, QMC::AccessPublic, 0x80000000 | 26, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'Reset'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(27, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'ResetBrightness'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(28, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'ResetChanges'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(29, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'SaveChanges'
        QtMocHelpers::SlotData<QDBusPendingReply<>()>(30, 2, QMC::AccessPublic, 0x80000000 | 15),
        // Slot 'SetBrightness'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &, double)>(31, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 17 }, { QMetaType::Double, 18 },
        }}),
        // Slot 'SetPrimary'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(32, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'SplitMonitor'
        QtMocHelpers::SlotData<QDBusPendingReply<>(const QString &)>(33, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::QString, 17 },
        }}),
        // Slot 'SwitchMode'
        QtMocHelpers::SlotData<QDBusPendingReply<>(short, const QString &)>(34, 2, QMC::AccessPublic, 0x80000000 | 15, {{
            { QMetaType::Short, 17 }, { QMetaType::QString, 18 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'Brightness'
        QtMocHelpers::PropertyData<BrightnessMap>(35, 0x80000000 | 36, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 0),
        // property 'BuiltinOutput'
        QtMocHelpers::PropertyData<QDBusObjectPath>(37, 0x80000000 | 38, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'DisplayMode'
        QtMocHelpers::PropertyData<short>(39, QMetaType::Short, QMC::DefaultPropertyFlags, 2),
        // property 'HasChanged'
        QtMocHelpers::PropertyData<bool>(40, QMetaType::Bool, QMC::DefaultPropertyFlags, 3),
        // property 'Monitors'
        QtMocHelpers::PropertyData<QList<QDBusObjectPath>>(41, 0x80000000 | 42, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'Primary'
        QtMocHelpers::PropertyData<QString>(43, QMetaType::QString, QMC::DefaultPropertyFlags, 5),
        // property 'PrimaryRect'
        QtMocHelpers::PropertyData<DisplayRect>(44, 0x80000000 | 45, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 6),
        // property 'ScreenHeight'
        QtMocHelpers::PropertyData<ushort>(46, QMetaType::UShort, QMC::DefaultPropertyFlags, 7),
        // property 'ScreenWidth'
        QtMocHelpers::PropertyData<ushort>(47, QMetaType::UShort, QMC::DefaultPropertyFlags, 8),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<DisplayInterface, qt_meta_tag_ZN16DisplayInterfaceE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject DisplayInterface::staticMetaObject = { {
    QMetaObject::SuperData::link<QDBusAbstractInterface::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DisplayInterfaceE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DisplayInterfaceE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN16DisplayInterfaceE_t>.metaTypes,
    nullptr
} };

void DisplayInterface::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<DisplayInterface *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->BrightnessChanged(); break;
        case 1: _t->BuiltinOutputChanged(); break;
        case 2: _t->DisplayModeChanged(); break;
        case 3: _t->HasChangedChanged(); break;
        case 4: _t->MonitorsChanged(); break;
        case 5: _t->PrimaryChanged(); break;
        case 6: _t->PrimaryRectChanged(); break;
        case 7: _t->ScreenHeightChanged(); break;
        case 8: _t->ScreenWidthChanged(); break;
        case 9: _t->__propertyChanged__((*reinterpret_cast<std::add_pointer_t<QDBusMessage>>(_a[1]))); break;
        case 10: { QDBusPendingReply<> _r = _t->Apply();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 11: { QDBusPendingReply<> _r = _t->AssociateTouchScreen((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 12: { QDBusPendingReply<> _r = _t->ChangeBrightness((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 13: { QDBusPendingReply<> _r = _t->JoinMonitor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 14: { QDBusPendingReply<QStringList> _r = _t->ListOutputNames();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QStringList>*>(_a[0]) = std::move(_r); }  break;
        case 15: { QDBusPendingReply<QString> _r = _t->QueryCurrentPlanName();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<QString>*>(_a[0]) = std::move(_r); }  break;
        case 16: { QDBusPendingReply<int> _r = _t->QueryOutputFeature((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<int>*>(_a[0]) = std::move(_r); }  break;
        case 17: { QDBusPendingReply<> _r = _t->Reset();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 18: { QDBusPendingReply<> _r = _t->ResetBrightness((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 19: { QDBusPendingReply<> _r = _t->ResetChanges();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 20: { QDBusPendingReply<> _r = _t->SaveChanges();
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 21: { QDBusPendingReply<> _r = _t->SetBrightness((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 22: { QDBusPendingReply<> _r = _t->SetPrimary((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 23: { QDBusPendingReply<> _r = _t->SplitMonitor((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        case 24: { QDBusPendingReply<> _r = _t->SwitchMode((*reinterpret_cast<std::add_pointer_t<short>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QDBusPendingReply<>*>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QDBusMessage >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::BrightnessChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::BuiltinOutputChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::DisplayModeChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::HasChangedChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::MonitorsChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::PrimaryChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::PrimaryRectChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::ScreenHeightChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (DisplayInterface::*)()>(_a, &DisplayInterface::ScreenWidthChanged, 8))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< BrightnessMap >(); break;
        case 6:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< DisplayRect >(); break;
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QDBusObjectPath >(); break;
        case 4:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QList<QDBusObjectPath> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<BrightnessMap*>(_v) = _t->brightness(); break;
        case 1: *reinterpret_cast<QDBusObjectPath*>(_v) = _t->builtinOutput(); break;
        case 2: *reinterpret_cast<short*>(_v) = _t->displayMode(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->hasChanged(); break;
        case 4: *reinterpret_cast<QList<QDBusObjectPath>*>(_v) = _t->monitors(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->primary(); break;
        case 6: *reinterpret_cast<DisplayRect*>(_v) = _t->primaryRect(); break;
        case 7: *reinterpret_cast<ushort*>(_v) = _t->screenHeight(); break;
        case 8: *reinterpret_cast<ushort*>(_v) = _t->screenWidth(); break;
        default: break;
        }
    }
}

const QMetaObject *DisplayInterface::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DisplayInterface::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN16DisplayInterfaceE_t>.strings))
        return static_cast<void*>(this);
    return QDBusAbstractInterface::qt_metacast(_clname);
}

int DisplayInterface::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDBusAbstractInterface::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}

// SIGNAL 0
void DisplayInterface::BrightnessChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void DisplayInterface::BuiltinOutputChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void DisplayInterface::DisplayModeChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void DisplayInterface::HasChangedChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void DisplayInterface::MonitorsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void DisplayInterface::PrimaryChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void DisplayInterface::PrimaryRectChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void DisplayInterface::ScreenHeightChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void DisplayInterface::ScreenWidthChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}
QT_WARNING_POP
